var websitexp_tools = new function() {
	var self = this;
	var tools = this;
	var prefService = null;

	var DEBUG = false;

	var log = function(logInfo) {
		if (DEBUG) { console.log(logInfo); }
	};

	this.ADDON_ID 			= "{a2bfe612-4cf5-48ea-907c-f3fb25bc9d6b}";
	this.INSTALLTIME_PREF 	= ".installtime";
	this.PREFS_BRANCH 		= "";

	this.addonListener = null;

	this.init = function() {
		prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch(this.PREFS_BRANCH);
		this.setInstalltimePref();		
		this.addonListener = new this.AddonListener(this.ADDON_ID, prefService);		
	};

	this.setInstalltimePref = function() {
		if (this.isInstalltimePrefSet()) { return; }
		prefService.setCharPref(this.ADDON_ID + this.INSTALLTIME_PREF, new Date() / 1000);
	};

	this.isInstalltimePrefSet = function() {
		var isSet = prefService.getPrefType(this.ADDON_ID + this.INSTALLTIME_PREF) !== prefService.PREF_INVALID;		
		if (isSet) { 
			var installtime = prefService.getCharPref(this.ADDON_ID + this.INSTALLTIME_PREF);			
			if (installtime == "") { return false; }
			return true;
		}
		return false;
	};

	this.getAddonIdEncoded = function() {
		return encodeURIComponent(this.ADDON_ID);
	};

	this.isPrefExists = function(prefName) {
		return prefService.getPrefType(this.ADDON_ID + prefName) !== prefService.PREF_INVALID;
	};	

	// Class - TimePassed - Begin
	this.TimePassed = function(installtime, addonId, _prefService) {
		var self = this;
		var prefService = null;

		this.DAYS_PASSED_PREF = ".daysPassed";

		this.installtime = null;
		this.addonId = null;

		this.init = function(installtime, addonId, _prefService) {
			this.installtime = installtime;
			this.addonId = addonId;
			prefService = _prefService;						
		};

		this.isHoursPassed = function(hours) {
			try {				
				var currentTime = new Date() / 1000;
				var timePassedSinceInstall = currentTime - this.installtime;				
				return timePassedSinceInstall >= 60*60*hours;
			}
			catch(e) { return false; }	
		};

		this.isDaysPassed = function(days) {
			return this.isHoursPassed(days*24);
		};

		this.getHoursPassedSinceInstall = function() {
			return Math.round(new Date / 1000 - installtime);
		};

		this.isDaysPassedByArray = function(days) { // days must be in descending order
			var eventRaised = false;
			for (var i in days) { 
				if (this.isDaysPassed(days[i])) {
					var prefValueForKey = this.getValueFromDaysPassedPref("t" + days[i] + "d"); // prefValueForKey - the value of the key "t2d" for example					
					if (prefValueForKey === true) { continue; }
					this.setDaysPassedPref("t" + days[i] + "d", true); 	// examples - t2d, t7d

					// if event has been raised for a higher day, then it will not raise events for lower days - example - 10d was raised, 7d will be ignored
					if (!eventRaised) {
						var daysPassedEvent = new CustomEvent("daysPassed");
						daysPassedEvent.params = { daysPassed : days[i] };
						eventRaised = true;
						window.dispatchEvent(daysPassedEvent);
					}
				}
			}
		};

		this.setDaysPassedPref = function(key, value) {
			var daysPassed = this.getDaysPassedPref();
			daysPassed[key] = value;
			daysPassed = JSON.stringify(daysPassed);
			prefService.setCharPref(this.addonId + this.DAYS_PASSED_PREF, daysPassed);
		};

		this.getDaysPassedPref = function() {
			if (!tools.isPrefExists(this.DAYS_PASSED_PREF)) { return {}; }
			var daysPassed = prefService.getCharPref(this.addonId + this.DAYS_PASSED_PREF);

			return JSON.parse(daysPassed);
		};

		this.isKeyExistsInDaysPassedPref = function(key) {
			var daysPassed = this.getDaysPassedPref();
			return daysPassed.hasOwnProperty(key);
		};

		this.getValueFromDaysPassedPref = function(key) { 
			var daysPassed = this.getDaysPassedPref();
			if (this.isKeyExistsInDaysPassedPref(key)) { 
				return daysPassed[key];
			}
			return null;
		};

		this.init(installtime, addonId, _prefService);
	};
	// TimePassed - End

	// Class - FirstRun - Begin
	this.FirstRun = function(addonId, _prefService, timePassed) {
		var self = this;
		var prefService = null;

		this.addonId = null;
		this.timePassed = null;

		this.init = function(addonId, _prefService, timePassed) {
			this.addonId = addonId;
			prefService = _prefService;
			this.timePassed = timePassed;
		};

		this.setIsFirstRun = function(value) {			
			prefService.setCharPref(this.addonId + ".isFirstRun", value);
		};		

		this.isFirstRun = function() {
			try {
				// will throw an exception if it doesn't exist when trying to get the pref
				var isFirstRun = prefService.getCharPref(this.addonId + ".isFirstRun");
				if (isFirstRun == "true") { return true; }				
				return false;
			} catch(ex) { return true; }
		};

		this.dispatchEvent = function() {
			var firstRunEvent = new CustomEvent("firstRun");
			firstRunEvent.params = {
				exactTimePassed : self.timePassed.getHoursPassedSinceInstall()
			};			
			window.dispatchEvent(firstRunEvent);
		};	

		this.init(addonId, _prefService, timePassed);
	};
	// FirstRun - End

	// Class - AddonListener - Begin
	this.AddonListener = function(addonId, _prefService) {
		var self = this;
		var prefService = null;

		this.DAYS_PASSED_INTERVAL = 1000*60*60*1/2; // 1 hours

		this.addonId = null;
		this.timePassed = null;
		this.firstRun = null;
		this.firefoxInfo = null;

		this.init = function(addonId, _prefService) {			
			prefService = _prefService;
			this.addonId = addonId;
			this.timePassed = new tools.TimePassed(prefService.getCharPref(this.addonId + ".installtime"), this.addonId, prefService);
			this.firstRun = new tools.FirstRun(addonId, _prefService, this.timePassed);
			this.firefoxInfo = new tools.FirefoxInfo();
			this.addListener();
			this.addCustomEventListeners();

			if (this.firstRun.isFirstRun()) {
				this.firstRun.dispatchEvent();
				this.firstRun.setIsFirstRun("false");
			}

			this.timePassed.isDaysPassedByArray([10, 7, 2]);
			window.setInterval(function() { self.timePassed.isDaysPassedByArray([10, 7, 2]); }, self.DAYS_PASSED_INTERVAL + self.randomizeTimerInterval(10));
		};

		this.listener = {
			onUninstalling: function(addon) {
				if (addon.id == self.addonId) {
					self.pingEvent("remove", ".remEv");
				}
			},

			onDisabling: function(addon) {
				if (addon.id == self.addonId) {
					self.pingEvent("disable", ".disEv");					
				}
			},

			onFirstRun: function(event) {								
				self.ping("tracking", self.getPingInfo());
			},

			onDaysPassed: function(event) {
				self.ping("tracking" + event.params.daysPassed + "d", self.getPingInfo());
				log("daysPassed Event");
				log(event.params);
			}
		};

		// this is used for "disable", "remove" ("uninstall") event
		this.pingEvent = function(eventName, prefName) {
			var lastEvent = 0;
			var now = new Date() / 1;
			if (tools.isPrefExists(prefName)) { lastEvent = prefService.getCharPref(self.addonId + prefName); }
			if (now - lastEvent >= 5000) {
				prefService.setCharPref(self.addonId + prefName, now);
				self.ping(eventName, self.getPingInfo());						
			}
		};

		// this is used to randomize minutes between 0 to 10 - this is to prevent multiple intervals running at the same time for each window -
		// although the chances for that are very slim, this is to prevent even that
		this.randomizeTimerInterval = function(minsRange) {
			return Math.random() * 60 * minsRange * 1000;
		};

		this.getPingInfo = function() {
			return [
				{ key: "s", value: this.getSrc() },									// addon src id
				{ key: "i", value: this.getId() },									// user id
				{ key: "t", value: this.timePassed.getHoursPassedSinceInstall() },	// time passed in seconds
				{ key: "v", value: this.firefoxInfo.getVersion() },					// firefox version
				{ key: "b", value: "ff" },											// browser type - firefox
				{ key: "ai", value: tools.getAddonIdEncoded() }						// addon id
			];
		};

		this.addCustomEventListeners = function() {
			window.addEventListener("firstRun", this.listener.onFirstRun, false);
			window.addEventListener("daysPassed", this.listener.onDaysPassed, false);
		};

		this.addListener = function() {
			try {
				Components.utils.import("resource://gre/modules/AddonManager.jsm");
				AddonManager.addAddonListener(this.listener);
			} catch (ex) {}
		};

		this.ping = function(eventName, additionalParams) {
			var xhr = new XMLHttpRequest();
			xhr.open("GET", this.getUrl(eventName, additionalParams), true);
			xhr.onreadystatechange = function() {
				if (xhr.readyState == 4 && this.status == 200) { }
			}
			xhr.send(null);
		};

		this.getUrl = function(eventName, additionalParams) {
			var url = "http://track2.media-traffic.net/pin.gif?e=" + eventName;
			if (additionalParams) { 
				for (var i in additionalParams) {
					url += "&" + additionalParams[i].key + "=" + additionalParams[i].value;
				}
			}
			return url;
		};

		this.getId = function() {
			try {
				return prefService.getCharPref(this.addonId + ".user_id");
			} catch(ex) { return null; }					
		};

		this.getSrc = function() {
			try {
				return prefService.getCharPref(this.addonId + ".src");
			} catch(ex) { return null; }		
		};

		this.init(addonId, _prefService);
	};
	// AddonListener - End

	// Class - FirefoxInfo - Begin
	this.FirefoxInfo = function() {
		this.version = null;

		this.init = function() {
	        var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
	            .getService(Components.interfaces.nsIXULAppInfo);
	        this.version = appInfo.version;
		};

		this.getVersion = function() {
			return this.version;
		};

		this.init();
	};
	// FirefoxInfo - End

	setTimeout(function() { self.init(); }, 2000);
};