var EXPORTED_SYMBOLS = ["FileCacher"];

var win = null;
var log = function(text) {
	if (win && DEBUG) { win.console.log(text); }
};

var DEBUG = false;

var FileCacher = function(fileUrl, localFilename, lastCachedPrefName) {
	var self = this;

	// consts
	this.DEFAULT_CACHE_REFRESH_TIME = 24;  // 24 hours
	this.DEBUG_CACHE_REFRESH_TIME = 0;
	this.LAST_CACHED_PREF_OBJECT_NAME = "lastC";

	// fields
	this.fileHandler = null;
	this.PrefMan = null;
	this.fileUrl = null;
	this.fileContents = null;
	this.cacheRefreshTime = null;
	this.lastCachedPrefName = null;

	this.init = function(fileUrl, localFilename, lastCachedPrefName) {
		this.setWindow();
		this.loadModules();
		this.setDebug();
		this.setCacheRefrestTime(this.DEFAULT_CACHE_REFRESH_TIME);
		this.lastCachedPrefName = lastCachedPrefName;
		var filename = this.getFilenameFromUrl(fileUrl);		
		this.fileUrl = fileUrl;		
		this.fileHandler = new FileHandler(localFilename);
	};

	this.setWindow = function() {
		var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
			.getService(Components.interfaces.nsIWindowMediator);
		win = wm.getMostRecentWindow(null);
	};

	this.loadModules = function() {
		Components.utils.import("resource://sitecounselor/PrefMan.js", this);
		Components.utils.import("resource://sitecounselor/AddonInfo.js", this);
	};

	this.setDebug = function() {
		if (DEBUG && this.AddonInfo && this.AddonInfo.debug === false) { DEBUG = false; }
	};

	this.setCacheRefrestTime = function(refreshTime) {
		this.cacheRefreshTime = refreshTime;
		if (DEBUG) { this.cacheRefreshTime = this.DEBUG_CACHE_REFRESH_TIME; }
	};

	this.getFileContents = function(callback) {
		function setFileContentsCallback(fileContents) { self.fileContents = fileContents; }

		var callbacksArray = [callback, setFileContentsCallback];
		// log(this);

		if (!this.fileHandler.isFileExist() || this.isTimeToRefresh()) {
			log("loadRemotely");
			this.setLastTimeRefreshedFilePref();
			this.loadRemotely(callbacksArray);
			return;
		}

		if (this.fileContents) {
			callback(this.fileContents); 
			log("read from memory");
			return;
		}

		this.fileHandler.readFile(callbacksArray);
	};

	this.isTimeToRefresh = function() {
		log("isTimeToRefresh");
		var lastCachedTimeStamp = this.PrefMan.getObjectPrefNameValue(this.LAST_CACHED_PREF_OBJECT_NAME, this.lastCachedPrefName);
		log(lastCachedTimeStamp);
		if (!lastCachedTimeStamp) { return true; }
		var now = Math.round(new Date() / 1000/60/60); // converts to hours
		return (now - lastCachedTimeStamp) >= this.cacheRefreshTime;
	};

	this.setLastTimeRefreshedFilePref = function() {
		log(this.lastCachedPrefName);
		this.PrefMan.setObjectPref(this.LAST_CACHED_PREF_OBJECT_NAME, this.lastCachedPrefName, Math.round(new Date() / 1000/60/60));
	};

	this.getFilenameFromUrl = function(url) {
		var paths = url.split("/");
		if (paths[paths.length - 1] === "") {
			return paths[paths.length - 2];
		}
		return paths[paths.length - 1];
	};

	this.changeFileExt = function(filename, newExt) {
		return filename.split(".")[0] + "." + newExt;
	};

	this.loadRemotely = function(callbacks) {
		log("load Remotly");
		var randomNumber = Math.trunc(Math.random()*10000000000);	// to prevent caching
		var xhr = new win.XMLHttpRequest();
		xhr.open("GET", this.fileUrl + "?r=" + randomNumber, true);
		xhr.setRequestHeader("Cache-Control", "no-cache, no-store, must-revalidate");
		xhr.setRequestHeader("Expires", "0");
		xhr.onreadystatechange = function() {
			if (xhr.readyState == 4 && xhr.status == 200) {
				self.fileHandler.writeToFile(xhr.responseText);
				self.fileContents = xhr.responseText;
				if (callbacks instanceof Array) {
					for (var i in callbacks) { 
						callbacks[i](xhr.responseText);
					}
					return;
				}
				callbacks(xhr.responseText);				
			}
		}
		xhr.send(null);
	};

	this.init(fileUrl, localFilename, lastCachedPrefName);
};

// FileHandler class
var FileHandler = function(_filename) {
	var self = this;	

	var filename = null;
	var XCipher = null;

	var IS_ENCRYPTED = true;

	this.init = function(_filename) {
		filename = _filename;
		this.loadModules();
		log(XCipher);
	};

	this.getModule = function(url, name) {
		var moduleContainer = {};
		Components.utils.import(url, moduleContainer);

		return moduleContainer[name];
	};

	this.loadModules = function() {
		XCipher = this.getModule("resource://sitecounselor/XCipher.js", "XCipher");
	};	

    this.getNsiFile = function(folder) {
    	folder = folder || "ProfD";

		var file = Components.classes["@mozilla.org/file/directory_service;1"].
			getService(Components.interfaces.nsIProperties).
			get(folder, Components.interfaces.nsIFile);
		file.append(filename);

		return file;
    };

	this.isFileExist = function() {
		log("is exists: " + this.getNsiFile().exists());
		return this.getNsiFile().exists();
	};

    this.writeToFile = function(data) {
		var file = this.getNsiFile();

        Components.utils.import("resource://gre/modules/NetUtil.jsm");
        Components.utils.import("resource://gre/modules/FileUtils.jsm");

        var ostream = FileUtils.openSafeFileOutputStream(file);
        var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].  
			createInstance(Components.interfaces.nsIScriptableUnicodeConverter);  
        converter.charset = "UTF-8";
        // if (IS_ENCRYPTED && !DEBUG) { 
        if (IS_ENCRYPTED) {
        	data = XCipher.encode("hounddog", data); 
    	}
        var istream = converter.convertToInputStream(data);
        log(data);
          
        NetUtil.asyncCopy(istream, ostream, function(status) {
            if (Components.isSuccessCode(status)) { }
            if (!Components.isSuccessCode(status)) { return; }
        });
    };

    this.readFile = function(onFileReadCallbacks) {
		var file = this.getNsiFile();

		Components.utils.import("resource://gre/modules/NetUtil.jsm");
		NetUtil.asyncFetch(file, function(inputStream, status) {
			if (!Components.isSuccessCode(status)) {
				// Handle error!
				return;
			}
			var data = NetUtil.readInputStreamToString(inputStream, inputStream.available());
			// if (IS_ENCRYPTED && !DEBUG) {
			if (IS_ENCRYPTED) {
        		data = XCipher.decode("hounddog", data);
			}
			// log(data);
			log("reading from file");
			win.setTimeout(function() {
				if (onFileReadCallbacks instanceof Array) {
					for (var i in onFileReadCallbacks) { 
						onFileReadCallbacks[i](data);
					}
					return;
				}
				onFileReadCallbacks(data);
			}, 1000);
		});
    };

    this.init(_filename);
};