var EXPORTED_SYMBOLS = ["PrefMan"];

var PrefMan = new function() {
	var prefBranch = null;	
	var prefTypes = null;
	var win = null;
	var addonInfo = null;

	// consts
	var PREFS_BRANCH = "";

	var log = function(text) {
		if (win) { win.console.log(text); }
	};

	this.init = function() {
		this.setWindow();
		addonInfo = this.getAddonInfo();
		var fullBranchString = (PREFS_BRANCH == "") ? addonInfo.id + "." : PREFS_BRANCH + "." + addonInfo.id + ".";
		prefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch(fullBranchString);
		this.setPrefTypes();
	};

	this.setWindow = function() {
		var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
			.getService(Components.interfaces.nsIWindowMediator);
		win = wm.getMostRecentWindow(null);
	};

	this.getAddonInfo = function() {
		var addonInfoModule = {};
		Components.utils.import("resource://sitecounselor/AddonInfo.js", addonInfoModule);

		return addonInfoModule.AddonInfo;
	};

	this.setPrefTypes = function() {
		prefTypes = {
			STRING 	: prefBranch.PREF_STRING,
			INT 	: prefBranch.PREF_INT,
			BOOL 	: prefBranch.PREF_BOOL,
			INVALID : prefBranch.PREF_INVALID
		};
	};

	this.getPrefTypes = function() {
		return prefTypes;
	};

	this.isPrefExists = function(prefName) {
		return prefBranch.getPrefType(prefName) !== prefBranch.PREF_INVALID;
	};

	this.setObjectPref = function(objectName, name, value) {	// no "type" param - type is always a string
		var pref = this.getObjectPref(objectName);
		if (!pref) { pref = {}; }

		pref[name] = value;	
		pref = JSON.stringify(pref);
		this.setPref(objectName, pref);
	};

	this.getObjectPref = function(objectName) {
		var pref = this.getPref(objectName);
		if (!pref) { return false; }

		try {
			pref = JSON.parse(pref);
			return pref;
		} catch(ex) { return false; }
	};

	this.getObjectPrefNameValue = function(objectName, name) {
		var pref = this.getObjectPref(objectName);
		if (!pref || !pref.hasOwnProperty(name)) { return false; }

		return pref[name];
	};

	this.setPref = function(name, value, type) {
		type = type || prefBranch.getPrefType(name);
		if (type === prefBranch.PREF_INVALID) { type = prefBranch.PREF_STRING; }

        switch(type) {
            case prefBranch.PREF_STRING :
                prefBranch.setCharPref(name, value);
                return true;
            break;
            case prefBranch.PREF_INT :
                prefBranch.setIntPref(name, value);
                return true;
            break;
            case prefBranch.PREF_BOOL :
                prefBranch.setBoolPref(name, value);
                return true;
            break;
            case prefBranch.PREF_INVALID :
                return false;
            break;		
        }			
	};

	this.getPref = function(name) {
		var type = prefBranch.getPrefType(name);
        switch(type) {
            case prefBranch.PREF_STRING :
                return prefBranch.getCharPref(name);
            break;
            case prefBranch.PREF_INT :
                return prefBranch.getIntPref(name);
            break;
            case prefBranch.PREF_BOOL :
                return prefBranch.getBoolPref(name);
            break;
            case prefBranch.PREF_INVALID :            	
                return false;
            break;		
        }			
	};

	this.init();
};